GOOGLE_URLS = {'GOOGLE': 'https://www.google.com/search?q={}&start={}', 
			   'GOOGLE_SCHOLAR': 'https://scholar.google.com/scholar?q={}&start={}', 
			   'GOOGLE_VIDEO': 'https://www.google.com/search?q={}&tbm=vid&start={}', 
			   'GOOGLE_BOOKS': 'https://www.google.com/search?q={}&tbm=bks&start={}',
			   'GOOGLE_NEWS': 'https://www.google.com/search?q={}&tbm=nws&start={}', 
			   'GOOGLE_SHOP': 'https://www.google.com/search?q={}&tbm=shop&start={}'
			  }

